

#define USE_NUMA //need 64 bit linux and numa libraries to use numa

#if defined(USE_NUMA) && defined(__linux__)
#include <numa.h>
#endif


int main(int argc, char **argv)
{
#ifdef __linux__
#ifdef USE_NUMA
  if (numa_available() < 0) {
    printf("Your system does not support NUMA API\n");
  }

  nodemask_t interleaveNodes;
  nodemask_zero(&interleaveNodes);

#ifndef NUM_BUILD_THREADS
  const int NUM_BUILD_THREADS = 1;
#endif
#ifndef NUM_RENDER_THREADS
  const int NUM_RENDER_THREADS = 0;
#endif

#if 0//optimize for grid build
  for (int thread=0; thread < NUM_BUILD_THREADS; ++thread)
    nodemask_set(&interleaveNodes, thread % (1+numa_max_node()));
#else //optimize for overall performance
  for (int thread=0; thread < NUM_BUILD_THREADS; ++thread)
    nodemask_set(&interleaveNodes, thread % (1+numa_max_node()));
  for (int thread=0; thread < NUM_RENDER_THREADS; ++thread)
    nodemask_set(&interleaveNodes, thread % (1+numa_max_node()));
#endif

  numa_set_interleave_mask(&interleaveNodes);

  const int NUM_PROCS = sysconf(_SC_NPROCESSORS_CONF);
  
  int cpu = 0;
  cpu_set_t mask;
  CPU_ZERO( &mask );
  CPU_SET( cpu, &mask );
  sched_setaffinity(0, sizeof(mask), &mask);

#else
  int cpu = 0;
  cpu_set_t mask;
  CPU_ZERO( &mask );
  CPU_SET( cpu, &mask );
  
  sched_setaffinity(0, sizeof(mask), &mask);
#endif //USE_NUMA

#endif //__linux__

  //at this point all memory allocations will be autmatically interleaved
  //across all nodes being used.

  //do_raytracing_stuff();

  return 0;
}
